@@ -23,15 +23,12 @@ static GMutex *object_lock = NULL;
 static guint
 g_CORBA_Object_hash (gconstpointer key)
 {
-	guint        retval;
 	CORBA_Object obj = (gpointer) key;
 
-	retval = obj->type_qid;
-
+	/* type_id is not reliable: cf. corbaloc */
 	g_assert (obj->object_key != NULL);
-	retval ^= IOP_ObjectKey_hash (obj->object_key);
 
-	return retval;
+        return IOP_ObjectKey_hash (obj->object_key);
 }
 
 static gboolean
@@ -250,48 +247,29 @@ static gboolean
 ORBit_try_connection (CORBA_Object obj)
 {
 	gboolean retval = FALSE;
-	gboolean disconnect = FALSE;
+	LinkConnectionStatus status;
 
-	if (giop_thread_safe ()) {
-		if (!obj->connection)
-			retval = FALSE;
-		else {
-			switch (link_connection_get_status
-					((LinkConnection *) obj->connection)) {
-			case LINK_CONNECTING:
-				dprintf (GIOP, "ORBit_try_connection bypassed while connecting ...");
-			case LINK_CONNECTED:
-				retval = TRUE;
-				break;
-			case LINK_DISCONNECTED:
-				disconnect = TRUE;
-				break;
-			}
-		}
-	} else {
-		switch (link_connection_wait_connected
-				((LinkConnection *) obj->connection)) {
-		case LINK_CONNECTING:
-			g_assert_not_reached();
-			break;
-		case LINK_CONNECTED:
-			retval = TRUE;
-			break;
-		case LINK_DISCONNECTED:
-			disconnect = TRUE;
-			break;
-		}
+	if (!obj->connection)
+		return FALSE;
+
+	status = link_connection_wait_connected
+		((LinkConnection *) obj->connection);
+
+	switch (status) {
+	case LINK_CONNECTING:
+		g_assert_not_reached();
+		break;
+	case LINK_CONNECTED:
+		retval = TRUE;
+		break;
+	case LINK_DISCONNECTED:
+		/* Have a go at reviving it */
+		dprintf (MESSAGES, "re-connecting dropped cnx %p", obj->connection);
+		giop_connection_try_reconnect (obj->connection);
+		break;
 	}
 
-	if (disconnect) {
-		if (obj->connection) {
-			giop_connection_unref (obj->connection);
-			obj->connection = NULL;
-		}
-		retval = FALSE;
-	}
-
-	return retval;
+	return TRUE;
 }
 
 GIOPConnection *
@@ -420,9 +398,14 @@ CORBA_Object_non_existent (CORBA_Object       obj,
 		return FALSE;
 
 	cnx = ORBit_object_get_connection (obj);
-	retval = cnx ? CORBA_FALSE : CORBA_TRUE;
-	if (cnx)
+
+	if (cnx) {
+		LinkConnectionStatus status;
+		status = link_connection_wait_connected (LINK_CONNECTION (cnx));
+		retval = (status == LINK_CONNECTED) ? FALSE : TRUE;
 		giop_connection_unref (cnx);
+	} else
+		retval = TRUE;
 
 	return retval;
 }
@@ -563,44 +546,66 @@ ORBit_demarshal_object (CORBA_Object   *obj,
 	return FALSE;
 }
 
-CORBA_boolean
-CORBA_Object_is_a (CORBA_Object       obj,
-		   const CORBA_char  *logical_type_id,
-		   CORBA_Environment *ev)
+CORBA_char*
+ORBit_object_to_corbaloc (CORBA_Object       obj,
+                          CORBA_Environment *ev)
 {
-	static GQuark  corba_object_quark = 0;
-	static GQuark  omg_corba_object_quark = 0;
-	CORBA_boolean  retval;
-	gpointer       args[] = { (gpointer *)&logical_type_id };
-	GQuark         logical_type_quark;
+        CORBA_char      *retval = NULL;
 
-	if (!corba_object_quark)
-		corba_object_quark = g_quark_from_static_string (
-			"IDL:CORBA/Object:1.0");
+        if (!obj) {
+                dprintf (OBJECTS, "Corbaloc NIL object\n");
+                return g_strdup ("corbaloc::/");
+        }
 
-	if (!omg_corba_object_quark)
-		omg_corba_object_quark = g_quark_from_static_string (
-			"IDL:omg.org/CORBA/Object:1.0");
+        OBJECT_LOCK (obj);
 
-	logical_type_quark = g_quark_from_string (logical_type_id);
+        if (!obj->profile_list) {
+                IOP_generate_profiles (obj);
+                ORBit_register_objref (obj);
+        }
 
-	if (logical_type_quark == corba_object_quark)
-		return CORBA_TRUE;
+        if (!(retval = ORBit_corbaloc_from (obj->profile_list,
+                                            obj->object_key))) {
+                CORBA_exception_set_system (
+                        ev, ex_CORBA_BAD_PARAM, CORBA_COMPLETED_NO);
+                /* FIXME, set minor code with vendor specific id */
+        }
+ 
+        OBJECT_UNLOCK (obj);
+         
+        return retval;
+}
 
-	if (logical_type_quark == omg_corba_object_quark)
-		return CORBA_TRUE;
 
-	if (!obj)
-		return CORBA_FALSE;
+CORBA_Object
+ORBit_object_by_corbaloc  (CORBA_ORB          orb,
+                           const gchar       *corbaloc,
+                           CORBA_Environment *ev)
+{
+        CORBA_Object  retval       = CORBA_OBJECT_NIL;
+        GSList       *profile_list = NULL;
 
-	if (logical_type_quark == obj->type_qid)
-		return CORBA_TRUE;
+        g_return_val_if_fail (orb!=NULL,      CORBA_OBJECT_NIL);
+        g_return_val_if_fail (corbaloc!=NULL, CORBA_OBJECT_NIL);
+        g_return_val_if_fail (ev!=NULL,       CORBA_OBJECT_NIL);
 
-	ORBit_small_invoke_stub (
-		obj, &CORBA_Object__imethods[4],
-		&retval, args, NULL, ev);
+        if (!(profile_list = ORBit_corbaloc_parse (corbaloc)))  {
+                CORBA_exception_set_system (
+                        ev, ex_CORBA_BAD_PARAM, CORBA_COMPLETED_NO);
+                /* FIXME, set minor code with vendor specific id */
+                return CORBA_OBJECT_NIL;
+        }
 
-	return retval;
+        if (!(retval = ORBit_objref_find (orb, "", profile_list))) {
+                CORBA_exception_set_system (
+                        ev, ex_CORBA_BAD_PARAM, CORBA_COMPLETED_NO);
+                /* FIXME, set minor code with vendor specific id */
+
+                IOP_delete_profiles (orb, &profile_list);
+                return CORBA_OBJECT_NIL;
+        }
+
+        return retval;
 }
 
 static gboolean
@@ -630,6 +635,50 @@ ORBit_impl_CORBA_Object_is_a(PortableServer_ServantBase *servant,
 	*(CORBA_boolean *)ret = ORBit_IInterface_is_a (ci->idata, type_id);
 }
 
+CORBA_boolean
+CORBA_Object_is_a (CORBA_Object       obj,
+		   const CORBA_char  *logical_type_id,
+		   CORBA_Environment *ev)
+{
+	static GQuark  corba_object_quark = 0;
+	static GQuark  omg_corba_object_quark = 0;
+	CORBA_boolean  retval;
+	gpointer       servant;
+	gpointer       args[] = { (gpointer *)&logical_type_id };
+	GQuark         logical_type_quark;
+
+	if (!corba_object_quark)
+		corba_object_quark = g_quark_from_static_string (
+			"IDL:CORBA/Object:1.0");
+
+	if (!omg_corba_object_quark)
+		omg_corba_object_quark = g_quark_from_static_string (
+			"IDL:omg.org/CORBA/Object:1.0");
+
+	logical_type_quark = g_quark_from_string (logical_type_id);
+
+	if (logical_type_quark == corba_object_quark)
+		return CORBA_TRUE;
+
+	if (logical_type_quark == omg_corba_object_quark)
+		return CORBA_TRUE;
+
+	if (!obj)
+		return CORBA_FALSE;
+
+	if (logical_type_quark == obj->type_qid)
+		return CORBA_TRUE;
+
+	if ((servant = ORBit_small_get_servant (obj)))
+		ORBit_impl_CORBA_Object_is_a (servant, &retval, args, NULL, ev, NULL);
+
+	else /* warning: obeys POA policies */
+		ORBit_small_invoke_stub (obj, &CORBA_Object__imethods[4],
+					 &retval, args, NULL, ev);
+
+	return retval;
+}
+
 static void
 ORBit_impl_ORBit_get_type_id (PortableServer_ServantBase *servant,
 			      gpointer ret, gpointer *args,
