@@ -39,6 +39,7 @@ static char        *orbit_ipname             = NULL;
 static char        *orbit_debug_options      = NULL;
 static char        *orbit_naming_ref         = NULL;
 static GSList      *orbit_initref_list       = NULL; 
+static GSList      *orbit_use_corbaloc       = FALSE;
 
 void
 ORBit_ORB_start_servers (CORBA_ORB orb)
@@ -253,6 +254,14 @@ shutdown_orb (void)
 	CORBA_exception_free (&ev);
 }
 
+static
+gboolean
+ORBit_initial_reference_protected_id (gchar* id)
+{
+        return (!strncmp (id, "RootPOA", strlen("RootPOA")) ||
+                !strncmp (id, "POACurrent", strlen("POACurrent")));
+}
+  
 static void 
 ORBit_initial_references_by_user (CORBA_ORB          orb, 
 				  gchar             *naming_ref,
@@ -298,8 +307,13 @@ ORBit_initial_references_by_user (CORBA_ORB          orb,
 			g_warning ("Option ORBInitRef has invalid object reference: %s=%s",  
 				   tuple->key, tuple->value);
 			CORBA_exception_free (ev);
-		} else
-			ORBit_set_initial_reference (orb, tuple->key, objref);
+		}  else if (ORBit_initial_reference_protected_id(tuple->key)) {
+                        g_warning ("Option ORBInitRef permission denied: %s=%s",
+
+                                   tuple->key, tuple->value);
+                } else {
+                        ORBit_set_initial_reference (orb, tuple->key, objref);
+                }
 	}
 
 }
@@ -324,8 +338,11 @@ CORBA_ORB_init (int *argc, char **argv,
 	/* the allocation code uses the bottom bit of any pointer */
 	g_assert (ORBIT_ALIGNOF_CORBA_DOUBLE > 2);
 
-	threaded = (orb_identifier &&
-		    strstr (orb_identifier, "orbit-local-mt-orb") != NULL);
+	if (orb_identifier &&
+	    strstr (orb_identifier, "orbit-local-non-threaded-orb") != NULL)
+		threaded = FALSE;
+	else
+		threaded = TRUE;
 
 	ORBit_option_parse (argc, argv, orbit_supported_options);
 	
@@ -391,6 +408,11 @@ CORBA_ORB_object_to_string (CORBA_ORB          orb,
 		return NULL;
 	}
 
+	/* FIXME, do nice integration */ 
+	if (orbit_use_corbaloc) {
+		return ORBit_object_to_corbaloc (obj, ev);
+	}
+	
 	buf = giop_send_buffer_use (orb->default_giop_version);
 
 	g_assert (buf->num_used == 1);
@@ -444,7 +466,12 @@ CORBA_ORB_string_to_object (CORBA_ORB          orb,
 	guchar              *tmpbuf;
 	int                  i;
 
-	if (strncmp (string, "IOR:", 4)) {
+	if (strncmp (string, "IOR:", strlen("IOR:"))            &&
+	    strncmp (string, "corbaloc:", strlen ("corbaloc:")) &&
+	    strncmp (string, "iiop:", strlen ("iiop:"))         &&
+	    strncmp (string, "iiops:", strlen ("iiops:"))       &&
+	    strncmp (string, "ssliop:", strlen ("ssliop:"))     &&
+	    strncmp (string, "uiop:", strlen ("uiop:"))) {
 
 #if defined ENABLE_HTTP
 		if (orbit_use_http_iors &&
@@ -467,41 +494,46 @@ CORBA_ORB_string_to_object (CORBA_ORB          orb,
 		}
 	}
 
-	string += 4;
-	len = strlen (string);
-	while (len > 0 && !g_ascii_isxdigit (string [len - 1]))
-		len--;
-
-	if (len % 2) {
-		if (ior)
-			g_free (ior);
-
-		return CORBA_OBJECT_NIL;
-	}
-
-	tmpbuf = g_alloca (len / 2);
-
-	for (i = 0; i < len; i += 2)
-		tmpbuf [i/2] = (g_ascii_xdigit_value (string [i]) << 4) |
-					g_ascii_xdigit_value (string [i + 1]);
-
-	buf = giop_recv_buffer_use_encaps (tmpbuf, len / 2);
-
-	if (ORBit_demarshal_object (&retval, buf, orb)) {
-		CORBA_exception_set_system (
+	if (!strncmp (string, "IOR:", strlen ("IOR:")))
+	{
+		string += 4;
+		len = strlen (string);
+		while (len > 0 && !g_ascii_isxdigit (string [len - 1]))
+			len--;
+		
+		if (len % 2) {
+			if (ior)
+				g_free (ior);
+			
+			return CORBA_OBJECT_NIL;
+		}
+		
+		tmpbuf = g_alloca (len / 2);
+		
+		for (i = 0; i < len; i += 2)
+			tmpbuf [i/2] = (g_ascii_xdigit_value (string [i]) << 4) |
+				g_ascii_xdigit_value (string [i + 1]);
+		
+		buf = giop_recv_buffer_use_encaps (tmpbuf, len / 2);
+		
+		if (ORBit_demarshal_object (&retval, buf, orb)) {
+			CORBA_exception_set_system (
 				ev,
 				ex_CORBA_MARSHAL,
 				CORBA_COMPLETED_NO);
-
-		retval = CORBA_OBJECT_NIL;
-	}
-
-	giop_recv_buffer_unuse (buf);
-
-	if (ior)
-		g_free (ior);
-
-	return retval;
+			
+			retval = CORBA_OBJECT_NIL;
+		}
+		
+		giop_recv_buffer_unuse (buf);
+		
+		if (ior)
+			g_free (ior);
+		
+		return retval;
+	} else {
+		return ORBit_object_by_corbaloc (orb, string, ev);
+	} 
 }
 
 void
@@ -1282,5 +1314,6 @@ const ORBit_option orbit_supported_options[] = {
 	{ "ORBSimpleUIDs",      ORBIT_OPTION_BOOLEAN, &orbit_use_genuid_simple },
 	{ "ORBDebugFlags",      ORBIT_OPTION_STRING,  &orbit_debug_options },
 	{ "ORBInitRef",         ORBIT_OPTION_KEY_VALUE,  &orbit_initref_list},
+	{ "ORBCorbaloc",        ORBIT_OPTION_BOOLEAN, &orbit_use_corbaloc},
 	{ NULL,                 0,                    NULL },
 };
